package com.uas.search.sort;

import com.uas.search.util.CharUtils;
import org.apache.lucene.search.FieldComparator;
import org.apache.lucene.search.FieldComparatorSource;

import java.io.IOException;

/**
 * 用于联想词的排序
 *
 * @author sunyj
 * @since 2017年7月3日 下午4:28:03
 */
public class SimilarValuesFieldComparatorSource extends FieldComparatorSource {
    @Override
    public FieldComparator<?> newComparator(String fieldname, int numHits, int sortPos, boolean reversed)
            throws IOException {
        return new DefaultFieldComparator(fieldname, numHits) {
            @Override
            protected int compare(String str1, String str2) {
                int superCompare = super.compare(str1, str2);
                if (superCompare != UNKOWN) {
                    return superCompare;
                }

                // 优先级：数字 > 字母 > 其他字符(优先级视为相等)
                if (str1.length() > str2.length()) {
                    return 1;
                } else if (str1.length() < str2.length()) {
                    return -1;
                }

                char[] array1 = str1.toCharArray();
                char[] array2 = str2.toCharArray();
                for (int i = 0; i < array1.length; i++) {
                    char c1 = array1[i];
                    char c2 = array2[i];
                    // 优先级：数字 > 字母 > 其他字符(优先级视为相等)
                    if (CharUtils.isNumber(c1)) {
                        if (CharUtils.isNumber(c2)) {
                            // 均为数字，数字越小优先级越高
                            if (c1 > c2) {
                                return 1;
                            } else if (c1 < c2) {
                                return -1;
                            }
                            continue;
                        }
                        return -1;
                    } else if (CharUtils.isCharacter(c1)) {
                        if (CharUtils.isNumber(c2)) {
                            return 1;
                        } else if (CharUtils.isCharacter(c2)) {
                            c1 = CharUtils.toUpper(c1);
                            c2 = CharUtils.toUpper(c2);
                            // 均为字母，值越小优先级越高
                            if (c1 > c2) {
                                return 1;
                            } else if (c1 < c2) {
                                return -1;
                            }
                            continue;
                        }
                        return -1;
                    }
                }
                return 0;
            }
        };
    }

}
